#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/**
	Function to calculate the magnitude of an image
	
	Fraga May 30, 2001
**/

#define logmod( a, b ) log(1.0 + sqrt( a*a + b*b ))
#define fmodulo( a, b ) sqrt( a*a + b*b )
#define fenergy( a, b ) a*a + b*b

void spectrum( float **imgin, float **imgModulo, int rows, int cols )
{
	#define Re(i,j)  imgin[i][2*(j)]
	#define Im(i,j)  imgin[i][2*(j)+1]

	int i, j, c2;
	float val;

	/* We going to calculate the modulo of the transform  */

	c2 = cols/2;

	for ( i=0; i<rows ; i++ )
		imgModulo[i][0] = logmod( imgin[i][0], imgin[i][1] );
		 
	for ( j=1; j<c2;  j++) { 
		val = logmod( Re(0,j), Im(0,j) );
		imgModulo[0][  j   ] = val;
		imgModulo[0][cols-j] = val;
	}
	for (i=1; i<rows ; i++) {
		for (j=1; j<c2;  j++) {
			val = logmod( Re(i,j), Im(i,j) );
			imgModulo[i][j] = val; 
			imgModulo[rows-i][cols-j] = val;
		}
	}
	for ( i=0; i<rows ; i++ )
		imgModulo[i][c2] = logmod( imgin[i][2*c2], imgin[i][2*c2 + 1] );

	#undef Re
	#undef Im
}

double total_energy( float **imgfft, int rows, int cols )
{
	#define Re(i,j)  imgfft[i][2*(j)]
	#define Im(i,j)  imgfft[i][2*(j)+1]

	int i, j, c2;
	double energy=0.0;

	/* We going to calculate the total energy of the Fourier transform  */

	c2 = cols/2;

	for ( i=0; i<rows ; i++ )
		energy += fenergy( imgfft[i][0], imgfft[i][1] );
		 
	for ( j=1; j<c2;  j++)  
		energy += 2.0 *( fenergy( Re(0,j), Im(0,j) ) );

	for (i=1; i<rows ; i++) {
		for (j=1; j<c2;  j++) {
			energy += 2.0 *( fenergy( Re(i,j), Im(i,j) ) );
		}
	}
	for ( i=0; i<rows ; i++ )
		energy += imgfft[i][2*c2] * imgfft[i][2*c2];
		/**energy += fenergy( imgfft[i][2*c2], imgfft[i][2*c2 + 1] ); **/

	return( energy );

	#undef Re
	#undef Im
}

double r_energy( float **imgfft, int rows, int cols, int r )
{
	int i, j;
	int c2=cols/2;
	int r2 = r*r;
	float cx, cy, x, y, y2;
	double energy=0.0;

	cy = (rows-1)/2.0;
	cx = (cols-1)/2.0;

	for (i=0; i<rows ; i++) {
		y = i - cy;
		y2 = y*y;

		x = - cx;  /** j=0 **/
		if ( x*x + y2 <= r2 ) {
			energy += fenergy( imgfft[i][0], imgfft[i][1] );
		}
		for (j=1; j<c2;  j++) {
			x = j - cx;
			if ( x*x + y2 <= r2 ) {
				energy += 2.0 * (fenergy( imgfft[i][2*j], imgfft[i][2*j + 1] ) );
			}
		}
		x = c2 - cx; /** j=c2 **/
		if ( x*x + y2 <= r2 )
			energy += imgfft[i][2*c2] * imgfft[i][2*c2];
			/** energy += fenergy( imgfft[i][2*c2], imgfft[i][2*c2 + 1] ); **/
	}
	return( energy );
}

void filter( float **img, int rows, int cols, float r, int type, int orden )
{
	int i, j, k;
	int c2=cols/2;
	float r2 = r*r;
	int r_1, r_2, r_12, r_22, radio2;

	float cx, cy, x, y, val, y2, aux, aux2;

	cy = (rows-1)/2.0;
	cx = (cols-1)/2.0;
	
	switch ( type ) {
		case 0:  /** Ideal Low pass **/
		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=c2;  j++) {
				x = j - cx;
				if ( (x*x + y2) > r2 ) {
					img[i][2*j] = 0; 
					img[i][2*j+1] = 0;
				}
			}
		}
		break;

		case 1:   /** Ideal High pass **/
		val = img[c2][2*c2];
		for (i=0; i<rows ; i++) {
			y = i - cx;
			y2 = y*y;
			for (j=0; j<=c2;  j++) {
				x = j - cx;
				if ( x*x + y2 <= r2 ) {
					img[i][2*j] = 0; 
					img[i][2*j+1] = 0;
				}
			}
		}
		img[c2][2*c2] = val;
		break;
		
		case 2: /** Raised Cosine Low Pass **/
		r_12 = rows/32;
		r_1 = r - r_12;
		r_1 = (r_1<0) ? 0 : r_1;
		r_2 = r + r_12;
		aux = M_PI/(r_2-r_1);
		r_12 = r_1*r_1;
		r_22 = r_2*r_2;

		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=c2;  j++) {
				x = j - cx;
				radio2 = x*x + y2;
				if ( radio2 < r_12 ); /** Do nothing **/ 
				else if (radio2 < r_22 ) {
					aux2 = (1.+ cos((sqrt(radio2)-r_1)*aux))*0.5;
					img[i][2*j] *= aux2; 
					img[i][2*j+1] *= aux2;
				}
				else {
					img[i][2*j] = 0; 
					img[i][2*j+1] = 0;
				}
			}
		}
		break;

		case 3: /** Raised Cosine High Pass **/
		r_12 = rows/32;
		r_1 = r - r_12;
		r_1 = (r_1<0) ? 0 : r_1;
		r_2 = r + r_12;
		aux = M_PI/(r_2-r_1);
		r_12 = r_1*r_1;
		r_22 = r_2*r_2;

		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=c2;  j++) {
				x = j - cx;
				radio2 = x*x + y2;
				if ( radio2 < r_12 ) {
					img[i][2*j] = 0; 
					img[i][2*j+1] = 0;
				}
				else if (radio2 < r_22 ) {
					aux2 = (1. - cos((sqrt(radio2)-r_1)*aux))*0.5;
					img[i][2*j] *= aux2; 
					img[i][2*j+1] *= aux2;
				}
			}
		}
		break;

		case 4: /** Gauss Low Pass **/
		r_12 = rows/32;
		r_1 = r - r_12;
		r_1 = (r_1<0) ? 0 : r_1;
		aux = (r-r_1)/0.8325546112; /** sqrt(-ln(-1.5)) **/
		r_12 = r_1*r_1;

		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=c2;  j++) {
				x = j - cx;
				radio2 = x*x + y2;
				if ( radio2 < r_12 ); /** Do nothing **/ 
				else {
					aux2 = (sqrt(radio2)-r_1)/aux;
					aux2 = exp(-aux2*aux2);
					img[i][2*j] *= aux2; 
					img[i][2*j+1] *= aux2;
				}
			}
		}
		break;

		case 5: /** Gauss High Pass **/
		r_12 = rows/32;
		r_1 = r - r_12;
		r_1 = (r_1<0) ? 0 : r_1;
		r_2 = r + r_12;
		aux = (r-r_1)/0.8325546112;
		r_22 = r_2*r_2;

		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=c2;  j++) {
				x = j - cx;
				radio2 = x*x + y2;
				if (radio2 < r_22 ) {
					aux2 = (sqrt(radio2)-r_2)/aux;
					aux2 = exp(-aux2*aux2);
					img[i][2*j] *= aux2; 
					img[i][2*j+1] *= aux2;
				}
			}
		}
		break;

		case 6: /** Fermi Low Pass **/
		aux = 0.01 * rows;

		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=c2;  j++) {
				x = j - cx;
				aux2 = (sqrt(x*x + y2)-r)/aux;

				aux2 = (aux2 <-10.0) ? -10.0 : aux2;
				aux2 = (aux2 > 10.0) ?  10.0 : aux2;
				aux2 = 1/(1 + exp(aux2));

				img[i][2*j] *= aux2; 
				img[i][2*j+1] *= aux2;
			}
		}
		break;

		case 7: /** Fermi High Pass **/
		aux = -0.01*rows;

		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=c2;  j++) {
				x = j - cx;
				aux2 = (sqrt(x*x + y2)-r)/aux;

				aux2 = (aux2 <-10.0) ? -10.0 : aux2;
				aux2 = (aux2 > 10.0) ?  10.0 : aux2;
				aux2 = 1/(1 + exp(aux2));

				img[i][2*j] *= aux2; 
				img[i][2*j+1] *= aux2;
			}
		}
		break;
		
		case 8: /** Butterworth Low Pass **/

		r_1 = orden; /** Orden Filter **/
		for (i=0; i<rows ; i++) {
			y = i - cy;
			y2 = y*y;
			for (j=0; j<=c2;  j++) {
				x = j - cx;

				aux = sqrt(x*x + y2)/r;

				for ( k=0, aux2=1.0; k<2*r_1; k++ ) aux2 *= aux;

				val = 1/sqrt(1 + aux2);

				img[i][2*j] *= val; 
				img[i][2*j+1] *= val;
			}
		}
		break;
	}
}
