/**
	Program to produce an normalized image 

	Input image must be in TIFF format

	Fraga. Jun 5, 2001
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "tiff.h"
#include "utils.h"

MATRIX * do_histo_normalization( MATRIX * IN ) 
{
	MATRIX *OUT, *HIST;
	float scale;
	byte s[256];
	int i, j, sum, *paux;
	byte *pin, *pout;
	int imin, imax;

	if ( (HIST = histogram( IN )) == NULL) return NULL;	
 	// print_histogram ( HIST );

	OUT = matrix_allocate( IN->rows, IN->cols, 1 );
	if ( OUT == NULL ) {
		fprintf( stderr,"ERROR allocating memory for output equalized image\n");
		return NULL;
	}

	paux = (int *)HIST -> ptr[0];
	for ( i=0; paux[i]==0 && i<256; i++ ); 
	imin = i;
	for ( i=255; paux[i]==0 && i>0; i-- ); 
	imax = i;

	printf( "# min=%d, max=%d\n", imin,  imax );

	if ( imin != imax ) scale = 255.0/(imax-imin);

	for ( i=imin; i<=imax; i++ ) {
		s[i] = (byte)(scale*(i-imin) + 0.5 );
	}
	for ( i=0; i<IN->rows; i++ ) {
		pin = (byte *)IN->ptr[i];
		pout = (byte *)OUT->ptr[i];

		for ( j=0; j<IN->cols; j++ ) 
			pout[j] = s [ pin[j] ]; 			
			// pout[j] = pin[j]; 			
	}

	return OUT;
}

int do_work( char *namein ) 
{
	MATRIX *IN, *EQ;
	FILEINFO finfo;
	int i, j;
	byte *iptr;
	

	IN = read_tiff( &finfo, namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in 256 grey tones\n");
		return -1;
	}

	EQ = do_histo_normalization( IN );
	if ( EQ == NULL ) {
		fprintf( stderr, "ERROR doing histogram equalization\n");
	 	return -1;
	}
	EQ->type=5;
	if ( write_tiff( EQ, "normalized.tif" ) != GOOD_WRITE )  
        return -1;

	matrix_free( IN );
	matrix_free( EQ );

	return 0;
}

int main( int argc, char *argv[] )
{
	char namein[32];

	if ( argc < 2 ) { 
		fprintf( stderr, "Use: %s image.tif\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );

	if ( do_work( namein ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
