/**
	Program to test the spatial filtering.

	Produce an output image (in TIFF format)
	resulted of convolution an input image and a small window
	(this is another small squared image of odd size, 3, 5, etc.)

	Fraga. Jun 11, 2001
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "pnm.h"


MATRIX *convolution_sobel( MATRIX *img, MATRIX *win )
{
	MATRIX *OUT;
	int i, j, k, l, s;
	int sum, val;
	byte *ptrin, *ptrout;
	int *ptrw;

	OUT = matrix_allocate( img->rows, img->cols, sizeof(byte) );
	if ( OUT==NULL ) {
		fprintf( stderr, "ERROR allocating memory for output image\n" );
		return NULL;
	}
	for ( i=0; i < img->rows; i++ ) {
		ptrout = (byte *) OUT->ptr[i];
		for ( j=0; j < img->cols; j++ ) 
			*ptrout++ = 0;
	}

	/** Suponemos que la ventana es cuadrada y de taqmao impar **/
	k = win -> rows / 2;

	for ( i=0; i <= img->rows - win->rows; i++ ) {
		ptrout = (byte *) OUT->ptr[i + k] + k;

		for ( j=0; j <= img->cols - win->cols; j++ ) {
			sum = 0;
			for ( l=0; l < win->rows; l++ ) {
				ptrin = (byte *)img->ptr[ i + l ] + j;
				ptrw = (int *)win->ptr[ l ];
				for ( s=0; s < win->cols; s++ ) 
					sum += *ptrin++ * *ptrw++;
			}
			val = (int)( abs(sum) + 0.5 );
			*ptrout++ = (val<0) ? 0 : (val>255) ? 255 : (byte)val;
		}
	}

	return OUT;	
}

void derivative( MATRIX *X, MATRIX *Y )
{
	byte *px, *py;
	int i, j, val;

	for ( i=0; i<X->rows; i++ ) {
		px = (byte *) X -> ptr[i];
		py = (byte *) Y -> ptr[i];
		for ( j=0; j<X->cols; j++ ) {
			val = ( *px > *py ) ? *px : *py;
			*px++ = (byte)val;
			py++;
		}
	}
}

int do_work( char *namein ) 
{
	MATRIX *IN, *OUTx, *OUTy, *FIL;
	int **fil;
	

	IN = read_pnm( namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in 256 grey tones\n");
		return -1;
	}
	FIL = matrix_allocate( 3, 3, sizeof( int ) );
	if ( FIL==NULL ) {
		fprintf( stderr, "ERROR allocating memory\n" );
		return -1;
	}
	fil = (int **)FIL->ptr;
	fil[0][0] =-1; fil[0][1] = 0; fil[0][2] = 1;
	fil[1][0] =-2; fil[1][1] = 0; fil[1][2] = 2;
	fil[2][0] =-1; fil[2][1] = 0; fil[2][2] = 1;


	OUTx = convolution_sobel( IN, FIL );
	if ( OUTx == NULL ) {
		fprintf( stderr, "ERROR doing convolution\n");
	 	return -1;
	}
	OUTx->type=5;
	if ( write_pnm( OUTx, "sobel_x.pgm" ) != 0 )  
        return -1;

	fil[0][0] =-1; fil[0][1] =-2; fil[0][2] =-1;
	fil[1][0] = 0; fil[1][1] = 0; fil[1][2] = 0;
	fil[2][0] = 1; fil[2][1] = 2; fil[2][2] = 1;
	OUTy = convolution_sobel( IN, FIL );
	if ( OUTy == NULL ) {
		fprintf( stderr, "ERROR doing convolution\n");
	 	return -1;
	}
	OUTy->type=5;
	if ( write_pnm( OUTy, "sobel_y.pgm" ) != 0 )  
        return -1;

	/** Calculamos la derivada **/

	derivative( OUTx, OUTy );
	if ( write_pnm( OUTx, "sobel.pgm" ) != 0 )  
        return -1;

	matrix_free( OUTx );
	matrix_free( OUTy );
	matrix_free( FIL );
	matrix_free( IN );

	return 0;
}

int main( int argc, char *argv[] )
{
	char namein[32];

	if ( argc < 2 ) { 
		fprintf( stderr, "Use: %s image.tif\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );

	if ( do_work( namein ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
