/**
	tiff.h
**/
/* Archivo de cabecera TIFF  */
/** Usados por los programas que leen y escriben 
	una imagen TIFF  **/

#include "matrix.h"

#define GOOD_READ     0         /* Cdigos de retorno */
#define GOOD_WRITE    0         /* Cdigos de retorno */
#define BAD_FILE      1
#define BAD_READ      2
#define MEMORY_ERROR  3
#define NO_DRIVER     4
#define BAD_ALLOC     5

#define ROJO          0
#define VERDE         1
#define AZUL          2
#define TAM           3

#define TIFFbyte      1         /* Tamanos del TIFF */
#define TIFFascii     2
#define TIFFshort     3
#define TIFFlong      4
#define TIFFrational  5

#define NewSubFile          254
#define SubfileType         255
#define ImageWidth          256
#define ImageLength         257
#define RowsPerStrip        278
#define StripOffsets        273
#define StripByteCounts     279
#define SamplesPerPixel     277
#define BitsPerSample       258
#define Compression         259
#define PlanarConfiguration 284
#define Group3Options       292
#define Group4Options       293
#define FillOrder           266
#define Threshholding       263
#define CellWidth           264
#define CellLength          265
#define MinSampleValue      280
#define MaxSampleValue      281
#define PhotometricInterp   262
#define GrayResponseUnit    290
#define GrayResponseCurve   291
#define ColorResponseUnit   300
#define ColorResponseCurve  301
#define XResolution         282
#define YResolution         283
#define ResolutionUnit      296
#define Orientation         274
#define DocumentName        269
#define PageName            285
#define XPosition           286
#define YPosition           287
#define PageNumber          297
#define ImageDescription    270
#define Make                271
#define Model               272
#define FreeOffsets         288
#define FreeByteCounts      289
#define ColorMap            320
#define Artist              315
#define DateTime            306
#define HostComputer        316
#define Software            305

#define II          0x4949
#define MM          0x4D4D

typedef struct {
	unsigned int width;
	unsigned int depth;
	unsigned int bytes;
	unsigned int widthbuffer; /** Ancho del buffer para la imagen **/
	unsigned int bits;
	unsigned int rowsperstrip;
	unsigned int count;
	unsigned int samples;
	unsigned int planarconfig;
	unsigned int compression;
	unsigned int bitspersample;
	unsigned int photometric;
	unsigned long bytecount;
	unsigned long offset;
	char palette[768];
} FILEINFO;

extern unsigned char masktable[8];

extern MATRIX * read_tiff( FILEINFO *fi, char *path);
extern int write_tiff( MATRIX *IN, char *path );
