#include <stdio.h>
#include "matrix.h"
#include "pnm.h"

/**
	Input: Image with float values
	Output: Image with grey tones (0-255)
**/
void rescale( float **imgin, byte **imgout, int rows, int cols )
{   
	float fmin, fmax, val, scale;
	int i, j;

	/** We rescale the dinamic range from 0 to 255 **/
	fmin = 1e30;
	fmax = -1e30;
	for ( i=0; i<rows; i++ ){
		for ( j=0; j<cols; j++ ){
			val = imgin[i][j]; 
			if ( fmin > val ) fmin = val;
			if ( fmax < val ) fmax = val;
		}   
	}
	printf( "max = %f\n", fmax );
	printf( "min = %f\n", fmin );

	scale = 255.0/(fmax-fmin);
	for ( i=0; i<rows; i++ ) {
		for ( j=0; j<cols; j++ ) {
			imgout[i][j] = (byte)(scale*( imgin[i][j] - fmin ) + 0.5 );
		}
	}
}

/** 
	Output: Return histogram in a MATRIX form: an image of
	        size 1x256 
	Input: Image in MATRIX structure
**/
MATRIX * histogram ( MATRIX * IN )
{
	MATRIX *HIST;
	int *histo, i, j;
	byte *iptr;

	HIST = matrix_allocate( 1, 256, sizeof(int) );
	if ( HIST==NULL ) {
		fprintf( stderr, "ERROR allocating memory for histogram\n" );
		return NULL;
	}
	histo = (int *)HIST->ptr[0];
	for ( i=0; i<256; i++ ) histo[i] = 0;

	for ( i=0; i<IN->rows; i++ ) {
		iptr = (byte *)IN->ptr[i];
		for ( j=0; j<IN->cols; j++ )
			histo[ *iptr++ ] ++;
	}
	return( HIST );
}

void print_histogram ( MATRIX * IN )
{
	int *histo, i;

	histo = (int *)IN->ptr[0];

	for ( i=0; i<256; i++ )
		printf( "%d %d\n", i,  histo[i] );
}


MATRIX * histogram_equalization( MATRIX * IN, int maxval ) 
{
	MATRIX *OUT, *HIST;
	float p_f[256];
	byte s[256];
	int i, j, sum, *paux;
	byte *pin, *pout;

	if ( (HIST = histogram( IN )) == NULL) return NULL;	
// 	print_histogram ( HIST );

	OUT = matrix_allocate( IN->rows, IN->cols, 1 );
	if ( OUT == NULL ) {
		fprintf( stderr,"ERROR allocating memory for output equalized image\n");
		return NULL;
	}

	paux = (int *)HIST -> ptr[0];
	sum = 0;
	for ( i=0; i<256; i++ ){
		sum += paux[i];
		p_f[i] = (float)sum/(IN->rows * IN->cols);
	}  
	for ( i=0; i<256; i++ ){
		s[i] = (byte )( (float)maxval*p_f[ i ] ); 			
	}

	for ( i=0; i<IN->rows; i++ ) {
		pin = (byte *)IN->ptr[i];
		pout = (byte *)OUT->ptr[i];

		for ( j=0; j<IN->cols; j++ ) 
			pout[j] = s [ pin[j] ]; 			
	}

	matrix_free( HIST );
	return OUT;
}


